#! usr/bin/python
# -*- coding: utf-8 -*-

from Tkinter import *
import ttk, tablelist, tkFileDialog, tkMessageBox
import ConfigParser, random, threading, os, urllib
import utils


class MyConfigParser (ConfigParser.ConfigParser):
    
    filename = ""
    
    def __init__ (self, filename):
        self.filename = filename
        ConfigParser.ConfigParser.__init__(self)
        ConfigParser.ConfigParser.read(self, self.filename)
    
    def set(self, section, option, value):
        ConfigParser.ConfigParser.set(self, section, option, value)
        ConfigParser.ConfigParser.write(self, open(self.filename, 'w'))

    def setboolean(self, section, option, value):
        try :
            if (bool(value) == True) :
                self.set(section, option, 'True')
            else :
                self.set(section, option, 'False')
        except :
            self.set(section, option, 'False')



class EASE_Manager :
    
    __window = None                     # Fenêtre
    
    __update = None                     # Instance de la classe de MAJ
    
    __current_release = None            # Release de la liste EASE
    __nb_total_annotations = None       # Nombre d'annotations au total dans la liste
    __nb_total_annotated_genes = None   # Nombre de gènes annotés dans la liste EASE (avec doublons)
    __table = None                      # Liste EASE actuelle
    __functional_class = None           # Annotation sélectionnée
    __title = None                      # Titre de la publication d'où est tirée l'annotation
    __pubmed = None                     # Lien PubMed de la publication
    __source = None                     # Détail sur la source de la liste des gènes pour l'annotation
    __nb_genes = None                   # Nombre de gènes annotés par l'annotation sélectionnée
    
    __thread = None                     # Thread du lancement de MAJ du serveur
    
    def __init__ (self):
        """Crée l'application et la fenêtre."""
        
        self.__update = Update_EASE_Manager()
                
            # Affichage de la fenêtre
        self.display_window()
        
        
    
    def __del__ (self):
        """Force la fermeture du thread, lors de la fermeture de l'application"""
        
        if (self.__thread != None) :
            self.__thread._Thread__stop()
    
    
    
    def display_window (self):
        """Affichage de la fenêtre."""
        
            # Création de la fenêtre
        self.__window = Tk()
        self.__window.title(LANG.get('EASE_MANAGER', 'window_title'))
        self.__window.resizable(width=False, height=False)
        
            # Conteneur général
        grille = Frame(self.__window)
        grille.pack(side=LEFT, padx=5, pady=5) 
        
            # Sélection de la liste EASE à afficher
        gr = Frame(grille)
        gr.grid(row=1, column=1, pady=5, sticky=W+E)
        
        gr2 = Frame(gr)
        gr2.grid(row=1, column=1, padx=20, sticky=W+E)
        
        self.__current_release = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'current_release')).grid(row=1, column=1, sticky=E)
        Entry(gr2, textvariable=self.__current_release, justify=CENTER, width=8, state='readonly').grid(row=1, column=2, sticky=W)

        gr2 = Frame(gr)
        gr2.grid(row=1, column=2, padx=20, sticky=W+E)     
        
        self.__nb_total_annotations = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'nb_total_annotations')).grid(row=1, column=1, sticky=E)
        Entry(gr2, textvariable=self.__nb_total_annotations, justify=CENTER, width=6, state='readonly').grid(row=1, column=2, sticky=W)

        gr2 = Frame(gr)
        gr2.grid(row=1, column=3, padx=20, sticky=W+E)     
        
        self.__nb_total_annotated_genes = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'nb_total_annotated_genes')).grid(row=1, column=1, sticky=E)
        Entry(gr2, textvariable=self.__nb_total_annotated_genes, justify=CENTER, width=8, state='readonly').grid(row=1, column=2, sticky=W)

        Button(gr, text=LANG.get('EASE_MANAGER', 'button_update'), command=self.__update_server).grid(row=1, column=4, padx=20)

            # Tableau : liste EASE
        gr = Frame(grille)
        gr.grid(row=2, column=1)    
            
        entete = [33, LANG.get('EASE_MANAGER', 'table_functional_class'), 12, LANG.get('EASE_MANAGER', 'table_first_author'),
                  7, LANG.get('EASE_MANAGER', 'table_date'), 30, LANG.get('EASE_MANAGER', 'table_title')]

        self.__table = tablelist.TableList(gr, background='white', columns=entete, height=11, stretch='all', width=100, labelcommand=self.__sort_by_column)
        self.__table.grid(row=1, column=1, pady=5, sticky=W+E)
        
        scrollbar = Scrollbar(gr, orient=VERTICAL)
        scrollbar.grid(row=1, column=2, pady=5, sticky=N+S)
        scrollbar.config(command=self.__table.yview)
        self.__table.config(yscrollcommand=scrollbar.set)   
        
            # Infos sur l'annotation
        gr = Frame(grille)
        gr.grid(row=3, column=1, pady=5)
        
        annot_infos = ttk.Labelframe(gr, text=" " + LANG.get('EASE_MANAGER', 'labelframe_info_annot') + " ")
        annot_infos.grid(row=1, column=1, padx=10, sticky=W)
        
        gr2 = Frame(annot_infos)    
        gr2.pack()
        
        self.__functional_class = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'label_functional_class')).grid(row=1, column=1, columnspan=3, padx=2, pady=2, sticky=W)
        Entry(gr2, textvariable=self.__functional_class, justify=LEFT, state='readonly', width=70).grid(row=2, column=1, columnspan=3, padx=2, pady=2, sticky=W+E)

        self.__title = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'label_title')).grid(row=3, column=1, columnspan=3, padx=2, pady=2, sticky=W)
        Entry(gr2, textvariable=self.__title, justify=LEFT, state='readonly', width=70).grid(row=4, column=1, columnspan=3, padx=2, pady=2, sticky=W+E)

        self.__pubmed = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'label_pubmed')).grid(row=5, column=1, padx=2, pady=2, sticky=W)
        Entry(gr2, textvariable=self.__pubmed, justify=LEFT, state='readonly').grid(row=5, column=2, columnspan=2, padx=2, pady=2, sticky=W+E)
       
        self.__source = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'label_source')).grid(row=6, column=1, padx=2, pady=2, sticky=W)
        Entry(gr2, textvariable=self.__source, justify=LEFT, state='readonly').grid(row=6, column=2, columnspan=2, padx=2, pady=2, sticky=W+E)
  
        self.__nb_genes = StringVar()
        Label(gr2, text=LANG.get('EASE_MANAGER', 'label_nb_genes')).grid(row=7, column=1, padx=2, pady=2, sticky=W)
        Entry(gr2, textvariable=self.__nb_genes, justify=LEFT, state='readonly', width=6).grid(row=7, column=2, padx=2, pady=2, sticky=W)
        
        Button(gr2, text=LANG.get('EASE_MANAGER', 'button_view_genes'), command=self.__display_genes_of_annotation_selected).grid(row=7, column=3, padx=10, pady=2, sticky=W)           
        
            # Boutons
        gr2 = Frame(gr)    
        gr2.grid(row=1, column=2, pady=5, sticky=W+E)

        Button(gr2, text=LANG.get('EASE_MANAGER', 'button_add_annotation'), height=2, width=20, command=self.__add_annotation).grid(row=1, column=1, padx=10, pady=10)           
        Button(gr2, text=LANG.get('EASE_MANAGER', 'button_edit_annotation'), height=2, width=20, command=self.__edit_annotation).grid(row=2, column=1, padx=10, pady=10)    
        Button(gr2, text=LANG.get('EASE_MANAGER', 'button_remove_annotation'), height=2, width=20, command=self.__remove_annotation).grid(row=3, column=1, padx=10, pady=10)    

            # Evénements 
        self.__table.bind('<<TablelistSelect>>', self.__show_details)

            # Vérification d'une MAJ
        res = self.__verify_updates()

            # Affichage de la liste EASE
        if (res == True) :
            self.__display_EASE_list()
            
        self.__window.mainloop()
                
        
        
    def __sort_by_column(self, table, col):
        """Tri du tableau selon la colonne sélectionnée."""
    
        order = '-increasing'
        if (self.__table.sortcolumn() == int(col)) and (self.__table.sortorder() == 'increasing') :
            order = '-decreasing'
        self.__table.sortbycolumn(col, order)
        
        
        
    def __verify_updates (self) :
        """Vérifie si une mise à jour de la liste EASE est disponible."""
        
            # Vérifie si une nouvelle version de la liste EASE est disponible
        # vérifie si une MAJ est nécessaire
        try :
            res = self.__update.check_updates_EASE_list()
        except :
            tkMessageBox.showerror("Server Error", LANG.get('EASE_MANAGER', 'err_server_check_list') % ("\n"))
            res = False
            return False
            
        # télécharge la MAJ si nécessaire
        if (res == True) :
            res = self.__update.update_EASE_list()      # téléchargement de la liste EASE
            if (res == False) :
                tkMessageBox.showerror("SERVER ERROR", LANG.get('EASE_MANAGER', 'err_server_download_list') % ("\n"))
            else :
                try :
                    tkMessageBox.showerror("SERVER", LANG.get('EASE_MANAGER', 'update_install_list'))
                    self.__update.update_EASE_program()              # installation de la liste EASE
                except :
                    tkMessageBox.showerror("ERROR", LANG.get('EASE_MANAGER', 'err_install_list'))
                    
        return True
        

        
    def __display_EASE_list (self):
        """Affichage de la liste EASE (+ informations)."""
        
        f = open('ease/Data/EASE_list_infos.txt', 'rb')

            # Affichage de la release, du nombre d'annotations et de gènes annotés
        line = f.readline()
        self.__current_release.set('WS ' + line.split('\t')[1].split(':')[0])
        self.__nb_total_annotations.set(line.split('\t')[3])
        self.__nb_total_annotated_genes.set(line.split('\t')[5])
        
            # Affichage de la liste EASE
        line = f.readline()
        while (line != "") :
            line = line.split('\t')
            self.__table.insert('end', (line[0], line[1], line[2], line[3]))
            line = f.readline()
        f.close()
        

        
    def __show_details (self, event=None):
        """Affichage des informations sur l'annotation sélectionnée."""
        
            # Récupère l'annotation sur laquelle on a cliqué
        select = self.__table.getcurselection()

            # Affiche toutes les informations sur l'annotation
        f = open('ease/Data/EASE_list_infos.txt', 'rb')
        
        line = f.readline()     # informations générales sur la liste (release, nb annotations, nb gènes annotés)
        line = f.readline()
        while (line != "") :
            line = line.strip().split('\t')
            if (line[0] == select[0][0]) and (line[1] == select[0][1]) and \
               (line[2] == select[0][2]) and (line[3] == select[0][3]) :
                self.__functional_class.set(line[0])
                self.__title.set(line[3])
                self.__pubmed.set(line[4])
                self.__source.set(line[5])
                self.__nb_genes.set(line[6])
                break
            line = f.readline()
        f.close()
        
        

    def __display_genes_of_annotation_selected (self):
        """Affichage de la liste des gènes possédant l'annotation sélectionnée."""
                
        def __save ():
            file = tkFileDialog.asksaveasfilename(filetypes = [("Gene list", "*.txt")])
            if (file != "") :
                if (file[-4:] != '.txt') : file = file + '.txt'
                
                    # Récupère la liste des gènes
                genes = gene_list.get(0, END)
                
                    # Enregistre la liste des gènes
                f = open(file, 'wb')
                for gene in genes :
                    f.write(gene + '\r\n')
                f.close()
            
            __win_genes_annotated.focus()
        
        
        def __copy ():
            genes = gene_list.get(0, END)                   # récupère la liste des gènes
            
            text = ""
            for gene in genes :
                text = text + gene + '\n'
                
            self.__window.clipboard_clear()
            self.__window.clipboard_append(text)            # enregistrement dans le presse-papiers
            
            
        def __quit ():
            __win_genes_annotated.destroy()        
        

            # Récupère l'annotation sélectionnée
        try :
            select = self.__table.getcurselection()
            annot = select[0][0] + ' ; ' + select[0][1] + ' ; ' + select[0][2]
        except :
            return                                          # si aucune annotation sélectionnée     
        
            # Création de la fenêtre
        __win_genes_annotated = Toplevel(self.__window)
        __win_genes_annotated.title(LANG.get('EASE_MANAGER_GENES_ANNOTATED', 'window_title'))
        __win_genes_annotated.resizable(width=False, height=False)
        
            # Objets
        grille = Frame(__win_genes_annotated)
        grille.pack(padx=5, pady=5)    
            
        annotation = StringVar()
        Label(grille, text=LANG.get('EASE_MANAGER_GENES_ANNOTATED', 'label')).grid(row=1, column=1, columnspan=3, padx=5, sticky=W)
        Entry(grille, textvariable=annotation, justify=LEFT, state='readonly').grid(row=2, column=1, columnspan=3, padx=5, pady=5, sticky=W+E)
        
        Button(grille, text=LANG.get('EASE_MANAGER_GENES_ANNOTATED', 'save'), width=10, command=__save).grid(row=3, column=1, padx=5)             
        Button(grille, text=LANG.get('EASE_MANAGER_GENES_ANNOTATED', 'copy'), width=10, command=__copy).grid(row=3, column=2, padx=5) 
 
        gene_list = Listbox(grille, height=10, width=30)
        gene_list.grid(row=4, column=1, columnspan=2, pady=5, sticky=N+S+W+E)
        scrollbar = ttk.Scrollbar(grille, orient=VERTICAL, command=gene_list.yview)
        scrollbar.grid(row=4, column=3, pady=5, sticky=N+S)
        gene_list['yscrollcommand'] = scrollbar.set

        Button(grille, text=LANG.get('EASE_MANAGER_GENES_ANNOTATED', 'close'), width=10, command=__quit).grid(row=5, column=1, columnspan=3, padx=5) 

            # Affichage de l'annotation
        annotation.set(annot)
        
            # Affichage de la liste des gènes possédant l'annotation
        f = open('ease/Data/EASE_list.txt', 'rb')
        line = f.readline()
        while (line != "") :
            line = line.strip().split('\t')
            try :
                   if (line[1][0] == '"') or (line[1][-1] == '"') :
                     line[1] = line[1][1:-1]
                     line[1] = line[1].replace('""', '"')
                   if (line[1] == annot) :
                       gene_list.insert(END, line[0])
            except :     # Exception si ligne vide
              pass
            line = f.readline()
        f.close()


        
    def __add_annotation (self):
        """Ajoute une annotation à la liste EASE."""     
        
        def __open ():
            file = tkFileDialog.askopenfilename(filetypes = [("Gene list", "*")])
            if (file != "") :
                    # Récupère le contenu du fichier
                f = open(file, 'rb')
                list = f.read()
                f.close()
                genes = list.replace('\r\n', '\n').split('\n')
                
                    # Ajout des gènes à la liste
                for gene in genes :
                    if (gene != "") : gene_list.insert(END, gene)
                    
            __win_add_annotation.focus()
        
        
        def __paste ():
                # Récupération du presse-papiers
            try :
                text = self.__window.clipboard_get()
            except :
                return
            
                # Séparation du texte copié pour récupérer la liste des gènes
            text = text.replace(',', '\n')
            text = text.replace(';', '\n')
            text = text.replace(' ', '\n')
            text = text.replace('\t', '\n')
            text = text.replace('/', '\n')
            text = text.replace('\r\n', '\n')
            genes = text.split('\n')
            
                # Ajout des gènes à la liste
            for gene in genes :
                if (gene != "") : gene_list.insert(END, gene)
        
        
        def __quit ():
            __win_add_annotation.destroy()
        
        
        def __add ():

                # Supprime les caractères supplémentaires en début et fin de ligne
            __functional_class.set(__functional_class.get().strip())
            __title.set(__title.get().strip())
            __author.set(__author.get().strip())
            __date.set(__date.get().strip())
            __pubmed.set(__pubmed.get().strip())
            __source.set(__source.get().strip())
            

                # Liste de gènes vide
            if (gene_list.size() == 0) : 
                tkMessageBox.showerror("ERROR", LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_no_gene'))
                __win_add_annotation.focus()
                return
                

                # Vérifie que les champs obligatoires ont bien été remplis
            if (__functional_class.get() == "") or (__author.get() == "") :
                tkMessageBox.showerror("ERROR", LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_no_functional_class_or_author'))
                __win_add_annotation.focus()
                return
            
            
                # Création de la date UNPUBLISHED si besoin
            if (__date.get() == "") : __date.set("Unpublished")


                # Récupère la liste des gènes
            genes = list(set(gene_list.get(0, END)))


                # Vérifie que les gènes sont bien au format WB ID
            for gene in genes :
                if (gene[0:6] != "WBGene") :
                    tkMessageBox.showerror(LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_wrong_id_title'), LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_wrong_id') % "\n")
                    __win_add_annotation.focus()
                    return


                # Vérifie que les IDs sont bien dans la dernière release
            genes_release = utils.get_geneIDs(int(self.__current_release.get()[3:]), ['wb'])
            not_recognized = [gene for gene in genes if gene not in genes_release['wb']]
            if (len(not_recognized) != 0) :
                tkMessageBox.showerror(LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_unknown_gene_title'), LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_unknown_gene') % (int(self.__current_release.get()[3:]), "\n"))
                __win_add_annotation.focus()
                return
                

                # Vérifie et télécharge la dernière version de la liste EASE + les infos sur la liste
            # vérifie si une MAJ est nécessaire
            try :
                res = self.__update.check_updates_EASE_list()
            except :
                tkMessageBox.showerror("Server Error", LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_server_check_list') % ("\n", "\n"))
                __win_add_annotation.focus()
                return
                
            # télécharge la MAJ si nécessaire
            if (res == True) :
                res = self.__update.update_EASE_list()      # téléchargement de la liste EASE
                if (res == False) :
                    tkMessageBox.showerror("SERVER ERROR", LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_server_download_list') % ("\n", "\n"))
                    __win_add_annotation.focus()
                    return
                    
                try :
                    tkMessageBox.showerror("SERVER", LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'update_install_list'))
                    self.update_EASE_program()              # installation de la liste EASE
                except :
                    tkMessageBox.showerror("ERROR", LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_install_list'))
                    __win_add_annotation.focus()
                    return
                

                # Vérifie que l'annotation n'existe pas déjà dans la liste
            f = open('ease/Data/EASE_list_infos.txt', 'rb')
            line = f.readline()     # infos sur la liste EASE
            line = f.readline()
            while (line != "") :
                line = line.strip().split('\t')
                if (line[0].lower() == __functional_class.get().lower()) and (line[1].lower() == __author.get().lower()) \
                   and (line[2].lower() == __date.get().lower()) :
                    f.close()
                    tkMessageBox.showerror(LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_annotation_already_exists_title'), LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'err_annotation_already_exists'))
                    __win_add_annotation.focus()
                    return
                line = f.readline()
            f.close()
        

                # Crée le fichier nécessaire à l'ajout de l'annotation
            random_number = str(random.randint(1,10000))
            f = open("temp/ADD_" + random_number + ".inf", 'wb')
            f.write('[INFORMATIONS]\r\n')
            f.write('functional_class = ' + __functional_class.get() + '\r\n')
            f.write('title = ' + __title.get() + '\r\n')
            f.write('author = ' + __author.get() + '\r\n')
            f.write('date = ' + __date.get() + '\r\n')
            f.write('pubmed = ' + __pubmed.get() + '\r\n')
            f.write('source = ' + __source.get() + '\r\n')
            f.write('\r\n[GENES]\r\n')
            f.write('0 = ' + str(len(genes)) + '\r\n')
            for i in range(len(genes)) :
                f.write(str(i+1) + ' = ' + genes[i] + '\r\n')
            f.close()
            
              
                # Envoi du fichier sur le serveur, et lancement de la MAJ
            thread = threading.Thread(target=self.__update.send_request_to_server, args=('ADD', "ADD_" + random_number + ".inf", "temp/ADD_" + random_number + ".inf"))
            thread.start()
                    
                # Message de confirmation de l'envoi de la demande d'ajout
            tkMessageBox.showinfo(LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'done_title'), LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'done') % "\n")

                # Suppression du fichier d'ajout temporaire
            os.remove("temp/ADD_" + random_number + ".inf")
            
            __win_add_annotation.focus()



            # Création de la fenêtre
        __win_add_annotation = Toplevel(self.__window)
        __win_add_annotation.title(LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'window_title'))
        __win_add_annotation.resizable(width=False, height=False)
        
            # Liste des gènes
        grille = Frame(__win_add_annotation)
        grille.pack(padx=5, pady=5)    
            
        gr = Frame(grille)
        gr.grid(row=1, column=1, padx=10)   
               
        Button(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'open_file'), width=8, command=__open).grid(row=2, column=1, padx=10)             
        Button(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'paste'), width=8, command=__paste).grid(row=2, column=2, padx=10) 
        Button(gr, text='X', command=lambda : gene_list.delete(0, END)).grid(row=2, column=3) 
 
        gene_list = Listbox(gr, height=14)
        gene_list.grid(row=3, column=1, columnspan=2, pady=5, sticky=N+S+W+E)
        scrollbar = ttk.Scrollbar(gr, orient=VERTICAL, command=gene_list.yview)
        scrollbar.grid(row=3, column=3, pady=5, sticky=N+S)
        gene_list['yscrollcommand'] = scrollbar.set
            
        gr = Frame(grille)
        gr.grid(row=1, column=2, padx=10)               
            
        __functional_class = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'functional_class')).grid(row=1, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__functional_class, justify=LEFT, width=35).grid(row=2, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __title = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'title')).grid(row=3, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__title, justify=LEFT, width=35).grid(row=4, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __author = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'author')).grid(row=5, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__author, justify=LEFT, width=35).grid(row=6, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __date = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'date')).grid(row=7, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__date, justify=LEFT, width=35).grid(row=8, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __pubmed = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'pubmed')).grid(row=9, column=1, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__pubmed, justify=LEFT).grid(row=9, column=2, padx=2, pady=2, sticky=W+E)
       
        __source = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'source')).grid(row=10, column=1, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__source, justify=LEFT).grid(row=10, column=2, padx=2, pady=2, sticky=W+E)
        
        Button(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'close'), width=10, command=__quit).grid(row=11, column=1, padx=10, pady=5) 
        Button(gr, text=LANG.get('EASE_MANAGER_ADD_ANNOTATION', 'add'), width=10, command=__add).grid(row=11, column=2, columnspan=2, padx=10, pady=5) 

        
        
    def __remove_annotation (self):
        """Supprime une annotation de la liste EASE."""         
        
            # Création de l'annotation telle qu'elle est dans la liste EASE
        try :
            select = self.__table.getcurselection()
            if (select == []) : raise
            annot = select[0][0] + ' ; ' + select[0][1] + ' ; ' + select[0][2]
        except :
            return                                          # si aucune annotation sélectionnée     
        
            # Récupère le nombre de gènes pour cette annotation
        nb_genes = self.__nb_genes.get()
        
        
            # Demande la confirmation de suppression de l'annotation
        res = tkMessageBox.askyesno(LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'remove_title'), LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'remove_warning') % ('"' + annot + '"\n'), icon=tkMessageBox.WARNING)
        if (res == False) : return
 
 
            # Demande du mot de passe pour la suppression
        passwd = StringVar()
        
        def __dial_cancel():
            passwd.set('<CANCEL>')
            dial.destroy()
 
        dial = Toplevel(self.__window)
        dial.title('')
        dial.resizable(width=False, height=False)
        gr = Frame(dial)
        gr.pack()
        Label(gr, text=LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'passwd_label')).grid(row=1, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        entry_passwd = Entry(gr, textvariable=passwd, width=25, show='*')
        entry_passwd.grid(row=2, column=1, columnspan=2, padx=5, pady=5)
        entry_passwd.focus()
        Button(gr, text=LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'passwd_ok'), width=10, command=dial.destroy).grid(row=3, column=1, padx=5, pady=7)
        Button(gr, text=LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'passwd_cancel'), width=10, command=__dial_cancel).grid(row=3, column=2, padx=5, pady=7)
        self.__window.wait_window(dial)

        if (passwd.get() == '<CANCEL>') or (passwd.get() == '') :
            return
        elif (passwd.get() != "admin_annot") :
            tkMessageBox.showerror(LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'err_passwd_title'), LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'err_passwd'))
            return
   
        
            # Crée la fichier nécessaire à la suppression de l'annotation
        random_number = str(random.randint(1,10000))
        f = open("temp/REM_" + random_number + ".inf", 'wb')
        f.write('[INFORMATIONS]\r\n')
        f.write('functional_class = ' + select[0][0] + '\r\n')
        f.write('author = ' + select[0][1] + '\r\n')
        f.write('date = ' + select[0][2] + '\r\n')
        f.write('nb_genes = ' + nb_genes + '\r\n')
        f.close()

        
            # Envoi du fichier sur le serveur, et lancement de la MAJ
        thread = threading.Thread(target=self.__update.send_request_to_server, args=('REM', "REM_" + random_number + ".inf", "temp/REM_" + random_number + ".inf"))
        thread.start()
        
            # Message de confirmation de l'envoi de la demande d'ajout
        tkMessageBox.showinfo(LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'done_title'), LANG.get('EASE_MANAGER_REMOVE_ANNOTATION', 'done') % "\n")
        
            # Suppression du fichier d'ajout temporaire
        os.remove("temp/REM_" + random_number + ".inf")
                    
        
        
    def __edit_annotation (self):
        """Modifie une annotation de la liste EASE."""         
        
        def __quit ():
            __win_edit_annotation.destroy()        
        
        def __edit ():

                # Supprime les caractères supplémentaires en début et fin de ligne
            __functional_class.set(__functional_class.get().strip())
            __title.set(__title.get().strip())
            __author.set(__author.get().strip())
            __date.set(__date.get().strip())
            __pubmed.set(__pubmed.get().strip())
            __source.set(__source.get().strip())
            

                # Vérifie que tous les champs ont bien été remplis
            if (__functional_class.get() == "") or (__author.get() == "") :
                tkMessageBox.showerror("ERROR", LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'err_no_functional_class_or_author'))
                __win_edit_annotation.focus()
                return
            
            
                # Création de la date UNPUBLISHED si besoin
            if (__date.get() == "") : __date.set("Unpublished")
        
         
                # Crée le fichier nécessaire à la modification de l'annotation
            random_number = str(random.randint(1,10000))
            f = open("temp/MOD_" + random_number + ".inf", 'wb')
            f.write('[OLD_ANNOTATION]\r\n')
            f.write('functional_class = ' + __functional_class__selected + '\r\n')
            f.write('title = ' + __title__selected + '\r\n')
            f.write('author = ' + __author__selected + '\r\n')
            f.write('date = ' + __date__selected + '\r\n')
            f.write('pubmed = ' + __pubmed__selected + '\r\n')
            f.write('source = ' + __source__selected + '\r\n')
            
            f.write('\r\n[NEW_ANNOTATION]\r\n')
            f.write('functional_class = ' + __functional_class.get() + '\r\n')
            f.write('title = ' + __title.get() + '\r\n')
            f.write('author = ' + __author.get() + '\r\n')
            f.write('date = ' + __date.get() + '\r\n')
            f.write('pubmed = ' + __pubmed.get() + '\r\n')
            f.write('source = ' + __source.get() + '\r\n')
            f.close()
            
              
                # Envoi du fichier sur le serveur, et lancement de la MAJ
            thread = threading.Thread(target=self.__update.send_request_to_server, args=('MOD', "MOD_" + random_number + ".inf", "temp/MOD_" + random_number + ".inf"))
            thread.start()
        
                # Message de confirmation de l'envoi de la demande de modification
            tkMessageBox.showinfo(LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'done_title'), LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'done') % "\n")
            
                # Suppression du fichier d'ajout temporaire
            os.remove("temp/MOD_" + random_number + ".inf")         
            
            __win_edit_annotation.destroy()


        
            # Vérifie qu'une annotation est bien sélectionnées
        try :
            select = self.__table.getcurselection()
            if (select == []) : raise
        except :
            return

            # Récupère les informtaions sur l'annotation sélectionnée
        __functional_class__selected = self.__functional_class.get()
        __title__selected = self.__title.get()
        __pubmed__selected = self.__pubmed.get()
        __source__selected = self.__source.get()
        __author__selected = select[0][1]
        __date__selected = select[0][2]
        
        
            # Création de la fenêtre
        __win_edit_annotation = Toplevel(self.__window)
        __win_edit_annotation.title(LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'window_title'))
        __win_edit_annotation.resizable(width=False, height=False)
        
            # Liste des gènes
        grille = Frame(__win_edit_annotation)
        grille.pack(padx=5, pady=5)    
            
        gr = Frame(grille)
        gr.grid(row=1, column=2, padx=10)               
            
        __functional_class = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'functional_class')).grid(row=1, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__functional_class, justify=LEFT, width=35).grid(row=2, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __title = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'title')).grid(row=3, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__title, justify=LEFT, width=35).grid(row=4, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __author = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'author')).grid(row=5, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__author, justify=LEFT, width=35).grid(row=6, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __date = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'date')).grid(row=7, column=1, columnspan=2, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__date, justify=LEFT, width=35).grid(row=8, column=1, columnspan=2, padx=2, pady=2, sticky=W+E)

        __pubmed = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'pubmed')).grid(row=9, column=1, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__pubmed, justify=LEFT).grid(row=9, column=2, padx=2, pady=2, sticky=W+E)
       
        __source = StringVar()
        Label(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'source')).grid(row=10, column=1, padx=2, pady=2, sticky=W)
        Entry(gr, textvariable=__source, justify=LEFT).grid(row=10, column=2, padx=2, pady=2, sticky=W+E)
        
        Button(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'close'), width=10, command=__quit).grid(row=11, column=1, padx=10, pady=5) 
        Button(gr, text=LANG.get('EASE_MANAGER_EDIT_ANNOTATION', 'edit'), width=10, command=__edit).grid(row=11, column=2, columnspan=2, padx=10, pady=5) 

            # Affichage des informations sur l'annotation sélectionnée
        __functional_class.set(__functional_class__selected)
        __title.set(__title__selected)
        __author.set(__author__selected)
        __date.set(__date__selected)
        __pubmed.set(__pubmed__selected)
        __source.set(__source__selected)
        
        
        
    def __update_server (self) :
        """Met à jour la liste EASE (vers la version freeze supérieure)."""     
        
            # Envoi de la demande de MAJ
        self.__thread = threading.Thread(target=self.__update.send_request_to_server, args=('UPDATE', None, None))
        self.__thread.start()
    
            # Message de confirmation de l'envoi de la demande de modification
        tkMessageBox.showinfo(LANG.get('EASE_MANAGER_UPDATE', 'done_title'), LANG.get('EASE_MANAGER_UPDATE', 'done') % "\n")
        
        
        
        
               
class Update_EASE_Manager :

    def __init__ (self) :
        pass
        
        
        
    def __download_server_file (self, path, filename):
        """Télécharge un fichier situé sur le serveur dans le dossier temporaire (temp/).
           path : Répertoire du serveur où est présent le fichier à télécharger 
           filename : Nom du fichier à télécharger"""
    
            # Télécharge le fichier
        if (path[-1] != '/') : path = path + '/'
        try :
            urllib.urlretrieve(path + filename, 'temp/' + filename)
        except :
            return False
        
            # Vérifie que le téléchargement est correct
        f = open('temp/' + filename, 'rb')
        line = f.readline().strip()
        if (line == '<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">') :
            f.close()
            os.remove('temp/' + filename)
            return False
        else :
            f.close()
            return True
        
        
        
    def __move_file (self, src, dest) :
        """Déplace un fichier.
           src & dest : Fichier source et de destination"""

        try :
            fr = open(src, 'rb')    # copie du fichier
            fw = open(dest, 'wb')
            fw.write(fr.read())
            fw.close()
            fr.close()

            os.remove(src)          # suppression du fichier source
        except :
            return False
        else : 
            return True        
        
        
        
    def check_updates_EASE_list (self):
        """Vérifie si une nouvelle liste EASE est disponible."""

            # Télécharge les informations du serveur
        res = self.__download_server_file(CFG.get('UPDATE', 'server_host'), 'config.ini')
        if (res == False) : raise
        
            # Récupère les informations de la dernière liste EASE
        ease_infos_cfg = utils.MyConfigParser('temp/config.ini')
        release_available = ease_infos_cfg.getint('EASE', 'release')
        version_available = ease_infos_cfg.getint('EASE', 'version')
        
            # Récupère les informations sur la release installée
        release_installed = CFG.getint('EASE', 'release')
        version_installed = CFG.getint('EASE', 'version')

            # Vérifie si la liste EASE a besoin d'être mise à jour
        if (release_available > release_installed) or ((release_available == release_installed) and (version_available > version_installed)) :
            return True
        else :
            return False        



    def update_EASE_list (self):
        """Télécharge et installe la nouvelle liste EASE."""
        
            # Télécharge la dernière version de la liste EASE
        res = self.__download_server_file(CFG.get('UPDATE', 'server_host') + 'ease/funcLists/', 'EASE_list.txt')
        if (res == False) : return False

            # Télécharge les informations sur la dernière version de la liste EASE
        res = self.__download_server_file(CFG.get('UPDATE', 'server_host') + 'ease/funcLists/', 'EASE_list_infos.txt')
        if (res == False) : return False

            # Si tous les téléchargements ont été correctement effectués, on déplace les fichiers dans le répertoire de EASE
        res1 = self.__move_file('temp/EASE_list.txt', 'ease/Data/EASE_list.txt')
        res2 = self.__move_file('temp/EASE_list_infos.txt', 'ease/Data/EASE_list_infos.txt')
        if (res1 == False) or (res2 == False) : return False

            # Mise à jour du fichier de configuration
        f = open('ease/Data/EASE_list_infos.txt', 'rb')
        infos = f.readline()    # récupère les infos sur la nouvelle liste
        f.close()
        
        CFG.set('EASE', 'release', infos.split('\t')[1].split(':')[0])
        CFG.set('EASE', 'version', infos.split('\t')[1].split(':')[1])
            
        return True
        
        

    def update_EASE_program (self):
        """Met à jour le programme EASE en modifiant les fichiers utilisés."""
        
            # Copie de la liste EASE (au format WB ID)
        fr = open('ease/Data/EASE_list.txt', 'rb')
        fw = open('ease/Data/Class/EASE_list (WB ID).txt', 'wb')
        fw.write(fr.read())
        fw.close()
        fr.close()
        
            # Récupère la version de la liste EASE
        f = open('ease/Data/EASE_list_infos.txt', 'rb')
        header = f.readline()
        f.close()
        
        release_used = int(header.split('\t')[1].split(':')[0])

            # Récupération de la liste des gènes de la release
        gene_list = utils.get_geneIDs(release_used, ['gene', 'wb'])
        
            # Création de la liste EASE au format Gene ID
        WB_list = []
        GeneID_list = []
            
        fr = open('ease/Data/Class/EASE_list (WB ID).txt', 'rb')
        fw = open('ease/Data/Class/EASE_list (Gene Sequence Name).txt', 'wb')
        
        line = fr.readline()
        while (line != "") :
            line = line.strip().split('\t')
            
            try :
                WB_list.append(line[0])
                index = gene_list['wb'].index(line[0])
                fw.write(gene_list['gene'][index] + '\t' + line[1] + '\r\n')
                GeneID_list.append(gene_list['gene'][index])
            except :
                pass
            
            line = fr.readline()
        fw.close()
        fr.close()
            
            # Création des listes de conversion
        f = open('ease/Data/Convert/WormBase ID.txt', 'wb')
        for gene in WB_list :
            f.write(gene + '\t' + gene + '\r\n')
        f.close()
        
        f = open('ease/Data/Convert/Gene Sequence Name.txt', 'wb')
        for gene in GeneID_list :
            f.write(gene + '\t' + gene + '\r\n')
        f.close()
        
            # Création des populations
        f = open('ease/Lists/Population (WB ID).txt', 'wb')
        for gene in WB_list :
            f.write(gene + '\r\n')
        f.close()
        
        f = open('ease/Lists/Population (Gene Sequence Name).txt', 'wb')
        for gene in GeneID_list :
            f.write(gene + '\r\n')
        f.close()
        
        return True
        
        
        
    def send_request_to_server(self, action, filename, file):
                    
            # Création des paramètres à envoyer      
        if (action != 'UPDATE') :
            f = open(file, 'rb')
            parametres = {'action' : action, 'filename' : filename, 'file' : f.read()}
            f.close()
        else :
            parametres = {'action' : action, 'filename' : "", 'file' : ""}
                                
            # Envoie le formulaire/requête avec les paramètres
        data = urllib.urlencode(parametres)
        f = urllib.urlopen(CFG.get('UPDATE', 'server_host') + 'ease_launcher.php', data)       

        
        
        
        
####################  DEMARRAGE DU SCRIPT  ####################
if __name__ == '__main__' :

        # Sélection de la langue
    CFG = MyConfigParser('config.ini')
    lang_file = CFG.get('WORMBASE', 'language')

    LANG = MyConfigParser(lang_file)

        # Création de l'application
    app = EASE_Manager()
            
